Metastatic.Supplemental.Error.ConflictError exception (Metastatic v0.22.0)

Raised when multiple supplementals attempt to handle the same construct.

Indicates ambiguity in which supplemental should transform a construct.