Metastatic.Supplemental.Error.ConflictError exception
(Metastatic v0.10.4)
Raised when multiple supplementals attempt to handle the same construct.
Indicates ambiguity in which supplemental should transform a construct.